import { Response, Headers, OptionsInit } from 'got';
import NodeCache from 'node-cache';
import { HttpClientConfig, IHttpClient, Logger, RequestOptions } from './interfaces';
export type HeaderFunction = () => Headers;
export declare class RequestException extends Error {
    readonly statusCode: string | number | undefined;
    readonly stack: string | undefined;
    readonly innerError: Error;
    constructor({ message, statusCode, innerError, stack, }: {
        message: string;
        statusCode?: string | number;
        innerError: Error;
        stack?: string;
    });
}
export type ProblemDetailsArgs = {
    detail?: string;
    instance?: string;
    statusCode: number;
    title: string;
    type: string;
    innerError: Error;
    extensions: {
        [k: string]: unknown;
    };
};
export declare class ProblemDetailsError extends RequestException {
    type: string;
    title: string;
    instance?: string;
    detail?: string;
    extensions: {
        [k: string]: unknown;
    };
    constructor(args: ProblemDetailsArgs);
}
export type HttpClientInitParams = {
    prefixUrl?: string;
    logger?: Logger;
    config?: HttpClientConfig;
    options?: OptionsInit;
    headerFunc?: HeaderFunction;
};
export declare class HttpClient implements IHttpClient {
    #private;
    constructor(initParams?: HttpClientInitParams);
    get<T>(path: string, options?: RequestOptions): Promise<T>;
    post<T>(path: string, data?: unknown, options?: Omit<RequestOptions, 'json'>): Promise<T>;
    put<T>(path: string, data?: unknown, options?: RequestOptions): Promise<T>;
    patch<T>(path: string, data?: unknown, options?: RequestOptions): Promise<T>;
    delete(path: string, options?: RequestOptions, data?: unknown): Promise<void>;
    /**
     * @description Access to all 'got' supported options, returns the response instead of JSON.
     * @param {string} path path or url
     * @param {OptionsInit} options all options supported by 'got'
     * @return {Promise<Response<T = unknown>>}
     */
    raw<T = unknown>(path: string, options: OptionsInit): Promise<Response<T>>;
}
type HttpMethodCall = 'get' | 'post' | 'put' | 'patch' | 'delete';
type RoutePayloads = {
    [p in HttpMethodCall]?: Record<string, unknown>;
};
export declare class TestHttpClient implements IHttpClient {
    _routePayloads: RoutePayloads;
    calls: Record<HttpMethodCall, Record<string, any>>;
    constructor(routePayloads: RoutePayloads);
    get<T>(route: string): Promise<T>;
    post<T>(route: string, data: Record<string, unknown>): Promise<T>;
    put<T>(route: string, data: Record<string, unknown>): Promise<T>;
    patch<T>(route: string, data: Record<string, unknown>): Promise<T>;
    delete<T>(route: string): Promise<T>;
    raw<T = unknown>(_path: string, _options: Omit<RequestOptions, 'json'>): Promise<Response<T>>;
    resetCalls(): void;
}
export interface ICachedHttpClient extends IHttpClient {
    getNoCache<T>(route: string): Promise<T>;
}
export declare class CachedClient implements ICachedHttpClient {
    #private;
    constructor(httpClient: HttpClient, logger: Logger, cache: NodeCache);
    get<T>(route: string): Promise<T>;
    getNoCache<T>(route: string): Promise<T>;
    post<T>(route: string, data?: Record<string, unknown>): Promise<T>;
    put<T>(route: string, data?: Record<string, unknown>): Promise<T>;
    patch<T>(route: string, data?: Record<string, unknown>): Promise<T>;
    delete(route: string): Promise<void>;
    raw<T = unknown>(path: string, options: Omit<RequestOptions, 'json'>): Promise<Response<T>>;
}
export {};
